import convnet from 'convnet';
import { predict, learn } from './src/layer/pool';
import gpuMock from 'gpu-mock.js';

const inputs = [[
  [13,2,3,4,5,6],
  [4,5,6,7,8,9],
  [7,8,9,10,11,12],
  [13,2,3,4,5,6],
  [4,5,6,7,8,9],
  [7,8,9,10,11,12]
]];
const context = {
  in_act: null,
  out_act: null,
  pad: 0,
  sx: 6,
  sy: 6,
  out_depth: 1,
  out_sx: 2,
  out_sy: 2,
  stride: 3,
  switchx: [],
  switchy: []
};

// convnet.PoolLayer.prototype.forward.call(context, toVol(inputs));
// context.out_act.dw = context.out_act.dw.map((value, i) => 9);
// convnet.PoolLayer.prototype.backward.call(context);
// process.exit(0);

const results1 = gpuMock(predict, {
  output: {
    x: 2,
    y: 2,
    z: 0
  },
  constants: {
    strideX: 3,
    strideY: 3,
    inputWidth: 6,
    inputHeight: 6,
    inputDepth: 1,
    paddingX: 0,
    paddingY: 0,
    filterWidth: 3,
    filterHeight: 3,
    filterCount: 1
  }
})(inputs);

console.log('');
const results2 = gpuMock(learn, {
  output: {
    x: 6,
    y: 6,
    z: 0
  },
  constants: {
    strideX: 3,
    strideY: 3,
    inputWidth: 6,
    inputHeight: 6,
    inputDepth: 1,
    outputWidth: 2,
    outputHeight: 2,
    paddingX: 0,
    paddingY: 0,
    filterWidth: 3,
    filterHeight: 3,
    filterCount: 1
  }
})([[9,9], [9,9]], [[0,3], [2,5]], [[0, 0],[5,5]]);
console.log('');
//console.log(to2DArray(context.out_act, true));
//console.log(context.out_act.dw);
console.log(results1);
console.log(results2);



function toVol(inputs) {
  const cInput = new convnet.Vol(inputs[0][0].length,inputs[0].length, inputs.length);
  for (let z = 0; z < inputs.length; z++) {
    for (let y = 0; y < inputs[z].length; y++) {
      for (let x= 0; x < inputs[z][y].length; x++) {
        cInput.set(x,y,z, inputs[z][y][x]);
      }
    }
  }
  return cInput;
}

function to3DArray(vol, getDw) {
  const depth = vol.depth;
  const height = vol.sy;
  const width = vol.sx;
  const grid = [];
  for (let z = 0; z < depth; z++) {
    const matrix = [];
    grid.push(matrix);
    for (let y = 0; y < height; y++) {
      const row = [];
      matrix.push(row);
      for (let x = 0; x < width; x++) {
        row.push(getDw ? vol.get(x, y, z) : vol.get_grad(x, y, z));
      }
    }
  }
  return grid;
}

function to2DArray(vol, getDw) {
  const height = vol.sy;
  const width = vol.sx;
  const matrix = [];
  for (let y = 0; y < height; y++) {
    const row = [];
    matrix.push(row);
    for (let x = 0; x < width; x++) {
      row.push(getDw ? vol.get(x, y, 0) : vol.get_grad(x, y, 0));
    }
  }

  return matrix;
}

function toArray(vol, getDw) {
  const width = vol.sy;
  const row = [];
  for (let x = 0; x < width; x++) {
    row.push(getDw ? vol.get(x, 0, 0) : vol.get_grad(x, 0, 0));
  }
  return row;
}