import convnet from 'convnet';
import gpuMock from 'gpu-mock.js';
import brain from './src';
const Dropout = brain.layer.Dropout;
import { trainingPredict, predict, learn } from './src/layer/dropout';


const inputs = [
  [-1,2,-3],
  [4,-5,6],
  [-7,8,-9]
];

const context = {
  constants: {
    probability: Dropout.defaults.probability
  }
};

const brainResults = gpuMock(trainingPredict, {
  output: [3, 3],
  constants: {
    probability: Dropout.defaults.probability
  }
})(inputs);

const that = convnetContext(context);
const results = convnet.DropoutLayer.prototype.forward.call(that, translateInputs2D(inputs), true);
that.dropped = that.dropped.map(() => true);
that.out_act.dw = that.out_act.dw.map(() => 0.5);
convnet.DropoutLayer.prototype.backward.call(that);
console.log(that.in_act.dw);

function convnetContext(context) {
  return {
    dropped: [],
    drop_prob: context.constants.probability,
    in_act: new convnet.Vol(3,3)
  };
}

function translateInputs2D(inputs) {
  const height = inputs.length;
  const width = inputs[0].length;
  const vol = new convnet.Vol(width, height, 1, 0);
  for (let y = 0; y < height; y++) {
    for (let x = 0; x < width; x++) {
      vol.set(x, y, 0, inputs[y][x]);
    }
  }
  return vol;
}

function translateInputs1D(inputs) {
  const width = inputs.length;
  const vol = new convnet.Vol(width, 1, 1, 0);
  for (let x = 0; x < width; x++) {
    vol.set(x, 0, 0, inputs[x]);
  }
  return vol;
}